/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.node;

public class NoCast implements Cast
{
  public final static NoCast instance = new NoCast();

  private NoCast()
  {}

  public Object cast(Object o)
  {
    return o;
  }
}
