/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.node;

public abstract class Token extends Node
{
  private String text;
  private int line;
  private int pos;

  public String getText()
  {
    return text;
  }

  public void setText(String text)
  {
    this.text = text;
  }

  public int getLine()
  {
    return line;
  }

  public void setLine(int line)
  {
    this.line = line;
  }

  public int getPos()
  {
    return pos;
  }

  public void setPos(int pos)
  {
    this.pos = pos;
  }

  public String toString()
  {
    return text + " ";
  }

  void removeChild(Node child)
  {}

  void replaceChild(Node oldChild, Node newChild)
  {}
}
