rem * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
rem * This file is part of SableCC.                             *
rem * See the file "LICENSE" for copyright information and the  *
rem * terms and conditions for copying, distribution and        *
rem * modification of SableCC.                                  *
rem * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
rem
rem IMPORTANT: Put the absolute path to sablecc.jar below.
rem
java -jar lib\sablecc.jar %1 %2 %3 %4 %5 %6 %7 %8 %9
