/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.node;

import org.sablecc.sablecc.analysis.*;

public final class TNull extends Token
{
  public TNull()
  {
    super.setText("Null");
  }

  public TNull(int line, int pos)
  {
    super.setText("Null");
    setLine(line);
    setPos(pos);
  }

  public Object clone()
  {
    return new TNull(getLine(), getPos());
  }

  public void apply(Switch sw)
  {
    ((Analysis) sw).caseTNull(this);
  }

  public void setText(String text)
  {
    throw new RuntimeException("Cannot change TNull text.");
  }
}
