/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.lexer;

public class LexerException extends Exception
{
  public LexerException(String message)
  {
    super(message);
  }
}
