/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.node;

import java.util.*;
import org.sablecc.sablecc.analysis.*;

public final class AProductionSpecifier extends PSpecifier
{

  public AProductionSpecifier()
  {}
  public Object clone()
  {
    return new AProductionSpecifier();
  }

  public void apply(Switch sw)
  {
    ((Analysis) sw).caseAProductionSpecifier(this);
  }

  public String toString()
  {
    return "";
  }

  void removeChild(Node child)
  {}

  void replaceChild(Node oldChild, Node newChild)
  {}
}
