/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.parser;

import java.util.ArrayList;

final class State
{
  int state;
  ArrayList nodes;

  State(int state, ArrayList nodes)
  {
    this.state = state;
    this.nodes = nodes;
  }
}
