/* This file was generated by SableCC (http://www.sablecc.org/). */

package org.sablecc.sablecc.node;

public class NodeCast implements Cast
{
  public final static NodeCast instance = new NodeCast();

  private NodeCast()
  {}

  public Object cast(Object o)
  {
    return (Node) o;
  }
}
